function [DW_all,SigTemp,ScoTemp,Correlations,W] =  ICA_DW_Corr_W(X,MaxICs)

% function [DW_all,SigTemp,ScoTemp,Correlations,W] =  ICA_DW_Corr_W(X,MaxICs);
% D.N. Rutledge, 03.03.2011
%
% Tries to optimise the number of ICs from a matrix of structured signlas,
% by computing th S/N ratio using Durbin-Watson
% for ICs extracted using models with increasing numbers of ICs

% Relative contribution of each IC is also calculated (using ICA scores)

% 1. maxICs ICA models are computed with from 1 to maxICs ICs
% 2. All maxICs(maxICs+1)/2 ICs are concatenated
% 3. The DW of each IC is calculated 
% 4. The angle between ICs of successive models is calculated to measure similarity
% 5. DW versus IC number is plotted with similar IC connected together
% 6. Proportion of each IC is calculated, using ICA scores.
% 7. Proportion versus IC number is plotted with similar IC connected together


% INPUT
% X         The analysed data matrix
% maxICs    The maximum number of ICs to calculate
%
% OUTPUT
% DW_all = Durbin-Watson criterion of all ICs
% SigTemp = matrix of concatenated IC signals
% ScoTemp = matrix of concatenated IC scores
% Correlations = structure with correlations between IC signals
% W = structure of IC unmixing matrices

% SUBROUTINES / PGMS CALLED
%
% jadeR_DNR00.m to calculate ICA
% Corr_DNR to calculate correlation between IC signals
% DurbWatMatrix (& DurbWatVector) to calculate DW values of IC signals


%% Do DW on ICA signals to choose number of ICs
DW_all=[];
for ICs=1:MaxICs
    
    Out=[num2str(ICs),'/',num2str(MaxICs)],;
    
%     [SigTemp{ICs},ScoTemp{ICs},old_X] =  jadeR_DNR00(X,ICs);
    [SigTemp{ICs},ScoTemp{ICs},W{ICs}] =  jadeR_DNR_W(X,ICs);

    for nIC=1:ICs
        [maxVal,maxInd]=max(abs(ScoTemp{ICs}(:,nIC)));
        if ScoTemp{ICs}(maxInd,nIC) < 0
            ScoTemp{ICs}(:,nIC)=-1*ScoTemp{ICs}(:,nIC);
            SigTemp{ICs}(:,nIC)=-1*SigTemp{ICs}(:,nIC);
        end
    end
end

%% Sort ICs base on successive correlations
for ICs=2:MaxICs
%     MaxAbsCorrs=max(abs(corr(SigTemp{ICs-1},SigTemp{ICs})),[],1);
    MaxAbsCorrs=max(abs(Corr_DNR(SigTemp{ICs-1},SigTemp{ICs})),[],1);
    
    temp1=[MaxAbsCorrs;SigTemp{ICs}]';
    temp2=[MaxAbsCorrs;ScoTemp{ICs}]';
    temp1 = sortrows(temp1,1);
    temp2 = sortrows(temp2,1);
    
    SigTemp{ICs}=temp1(:,2:end)';
    ScoTemp{ICs}=temp2(:,2:end)';
end;

%% Do DW on IC to choose number of ICs
DW_all=[];
Prop_A2_all=[];

for ICs=1:MaxICs
    %[DW] = DurbWatMatrix('col',SigTemp{ICs});
    [DW] = DurbWatVector(SigTemp{ICs});
    DW_all=[DW_all,DW];
    
    [Prop_A2] = sum(ScoTemp{1,ICs}.*ScoTemp{1,ICs},1);
    Prop_A2_Toto=sum(Prop_A2);
    
    % Like ANOVA ?
    Prop_A2=Prop_A2./Prop_A2_Toto;

      Prop_A2_all=[Prop_A2_all,Prop_A2];
end

nICs=size(DW_all,2);

%% Calculate correlations between ICA Signals
% Find successive pairs with maximum correlation
for ICs=2:MaxICs
    Correlations{ICs}=corr(SigTemp{ICs-1},SigTemp{ICs});
    
    for i=1:ICs-1
        % 'abs' may not be necessary
        [C,Ind{ICs}(i,:)] = max(abs(Correlations{ICs}(i,:)),[],2);
    end;
end;

%%
% Seq1=[1,2,4,7,11,16,22,29,37,46]';
Seq1(1,1)=1;
for i=1:MaxICs-1
    Seq1(i+1,1)=Seq1(i,1)+i;
end

Seq1=Seq1-1;

%% Prepare IC numbers
S=[];
for i=1:MaxICs
    S=[S;[1:i]'];
end;
strS=num2str(S);

Individ=[1:nICs];

%% Plot DWs
figure;
plot(DW_all,'r:x'), axis tight;
%text(Individ,double(DW_all),strS);

hold on;

for j=1:MaxICs-1
    toto(1:j-1,j)=1;
    toto(j,j)=1;
    if j>1
        if Ind{1,i}(toto(j,j),1)==Ind{1,i}(toto(j,j-1),1)
            toto(j,j)=2;
        end;
    end;

    for i=j+1:MaxICs
        toto(i,j)=Ind{1,i}(toto(i-1,j),1);
    end;
    tutu(:,j)=Seq1+toto(:,j);
    plot(tutu(:,j),DW_all(1,tutu(:,j)),'k-O');
end

title('Durbin-Watson on IC signals');

%% Plot log(DW)
figure;
plot(log(DW_all),'r:x'), axis tight;
text(Individ,double(log(DW_all)),strS);
hold on;

for j=1:MaxICs-1
    toto(1:j-1,j)=1;
    toto(j,j)=1;
    if j>1
        if Ind{1,i}(toto(j,j),1)==Ind{1,i}(toto(j,j-1),1)
            toto(j,j)=2;
        end;
    end;

    for i=j+1:MaxICs
        toto(i,j)=Ind{1,i}(toto(i-1,j),1);
    end;
    tutu(:,j)=Seq1+toto(:,j);
    plot(tutu(:,j),log(DW_all(1,tutu(:,j))),'k-O');
end

title('Log(Durbin-Watson) on IC signals');

%% Plot Prop_A2_all
figure;
plot(Prop_A2_all,'r:x'), axis tight;
text(Individ,double(Prop_A2_all),strS);
hold on;

for j=1:MaxICs-1
    toto(1:j-1,j)=1;
    toto(j,j)=1;
    if j>1
        if Ind{1,i}(toto(j,j),1)==Ind{1,i}(toto(j,j-1),1)
        toto(j,j)=2;
        end;
    end;
    
    for i=j+1:MaxICs
        toto(i,j)=Ind{1,i}(toto(i-1,j),1);
    end;
    tutu(:,j)=Seq1+toto(:,j);
    plot(tutu(:,j),Prop_A2_all(1,tutu(:,j)),'k-O');
end
title('Prop''n of ICs in X');

%% Plot log(Prop_A2_all)
figure;
plot(log(Prop_A2_all),'r:x'), axis tight;
text(Individ,double(log(Prop_A2_all)),strS);
hold on;

for j=1:MaxICs-1
    toto(1:j-1,j)=1;
    toto(j,j)=1;
    if j>1
        if Ind{1,i}(toto(j,j),1)==Ind{1,i}(toto(j,j-1),1)
        toto(j,j)=2;
        end;
    end;
    
    for i=j+1:MaxICs
        toto(i,j)=Ind{1,i}(toto(i-1,j),1);
    end;
    tutu(:,j)=Seq1+toto(:,j);
    plot(tutu(:,j),log(Prop_A2_all(1,tutu(:,j))),'k-O');
end
title('log(Prop''n of IC in X)');
