function [Signal,Scores] =  jade_DNR(X,ICs, Options);

% INPUT variables
%
%   X(nxT) = mixed signal dataset
%   ICs=  Sources (pure signal components) =underlying signals =pure spectra
%   where :
%   n=  Sensors (mixed signal inputs) = rows =spectra
%   T=  Signal data points = columns =variables
%
%   Options.Method = 'Tall', 'Wide', 'Kernel', 'ComDim', 'Normal' X
%   Options.Partitions = number of partition if X tall or wide
%   Options.Centred = Matrix centred by column
%
%
% OUTPUT variables
%   Signal(nxT) or Signal(ICsxT)  = pure signal dataset
%   Scores = "scores" of samples calculated from ICs
%          = old_X*Signal*inv(Signal'*Signal);
%
%
% CALLS
% jadeR_2005
% ColumnsPartition
% PCA_Tall_PCT_DNR
%   which calls
%   RowsPartition

[nR,nC]	= size(X);
MaxRank=min(nR,nC);

old_X=X;

if exist('Options','var')
    if isfield(Options,'Centred')
        switch Options.Centred
            case 1 % X matrix centred
                X= X-mean(X)*ones(nC,1);
                old_X=X;
            otherwise % X matrix NOT centred
                old_X=X;
        end
    else
        old_X=X;
    end
end


if exist('Options','var')

    if isfield(Options,'Method')
        switch Options.Method
            case 'Tall'
                if isfield(Options,'Partitions')
                    partitions=Options.Partitions;
                else
                    Options.Partitions=1;
                    partitions=Options.Partitions;
                end

                %% Do Tall segmented PCT
                [t] = PCA_Tall_PCT_DNR(X, MaxRank, partitions);

                [u,s,v]=svd(t, 'econ');

                % Calculate PCT Loadings
                Techelle=inv(u'*u)*u';
                Vpct=Techelle*old_X;

                B =  jadeR_2005(Vpct,ICs);

                % Calculate pure signals from unmixing matrix and X
                Signal=(B*Vpct)';

            case 'Wide'
                if isfield(Options,'Partitions')
                    partitions=Options.Partitions;
                else
                    Options.Partitions=1;
                    partitions=Options.Partitions;
                end

                Xi = ColumnsPartition(X, partitions);
                T_all=[];
                for i = 1:partitions
                    [U_in,S_in,V_in]= svd(Xi{i}, 'econ');
                    minICS=min(ICs,size(U_in,2));
                    T_all=[T_all U_in];
                end
                %% Do PCT on concatenated Scores of Segments
                [u,s,v]= svd(T_all, 'econ');

                %% Calculate PCT Loadings
                Techelle=inv(u'*u)*u';
                Vpct=Techelle*old_X;

                B =  jadeR_2005(Vpct,ICs);

                % Calculate pure signals from unmixing matrix and X
                Signal=(B*Vpct)';

            case 'Kernel'
                if nR>nC
                    Kernel=(X'*X)/nC;

                    [u,s,v]= svd(Kernel,'econ')	;
                    D=s*s;
                    % D= Eigenvalues

                    Ut=u; % Ut here == v from SVD on X
                    % Ut= Eigenvectors

                    [puiss,k]= sort(diag(D),'descend')	;
                    rangeW= 1:ICs			;   % indices to the ICs  most significant directions

                    Ut=Ut(:,k(rangeW));

                    W=(X*Ut)';% whitener
                    X= W*old_X;

                    B =  jadeR_2005(X,ICs);

                    % Calculate pure signals from unmixing matrix and X
                    Signal=(B*X)';
                else
                    Kernel=(X*X')/nR;

                    [u,s,v]= svd(Kernel,'econ')	;
                    D=s*s;
                    % D= Eigenvalues

                    Ut=u; % Ut here == v from SVD on X
                    % Ut= Eigenvectors

                    [puiss,k]= sort(diag(D),'descend')	;
                    rangeW= 1:ICs			;   % indices to the ICs  most significant directions

                    Ut=Ut(:,k(rangeW));

                    %% Calculate PCT Loadings
                    Techelle=inv(Ut'*Ut)*Ut';
                    Vpct=Techelle*old_X;

                    B =  jadeR_2005(Vpct,ICs);

                    % Calculate pure signals from unmixing matrix and X
                    Signal=(B*Vpct)';
                end

            case 'ComDim'
                if isfield(Options,'Partitions')
                    partitions=Options.Partitions;
                else
                    Options.Partitions=1;
                    partitions=Options.Partitions;
                end

                Xi = ColumnsPartition(X, partitions);

                % Do Segmented PCT based on ComDim trick
                % SVD on matrix of sums of X.X'
                XiXiT=zeros(nR,nR);
                for i = 1:partitions
                    XiXiT=XiXiT+Xi{i}*Xi{i}';
                end

                % Do PCT on summed Sample-based variance matrices
                [u,s,v]= svd(XiXiT, 'econ');

                % Calculate PCT Loadings
                Techelle=inv(u'*u)*u';
                X=Techelle*old_X;

                B =  jadeR_2005(X,ICs);

                % Calculate pure signals from unmixing matrix and X
                Signal=(B*X)';

            case 'Normal'
                B =  jadeR_2005(X,ICs);

                % Calculate pure signals from unmixing matrix and X
                Signal=(B*old_X)';

        end
    else
        Options.Method='Normal';
        B =  jadeR_2005(X,ICs);

        % Calculate pure signals from unmixing matrix and X
        Signal=(B*old_X)';

    end

end

% Calculate IC_Scores (proportions) of Jade
Scores=old_X*Signal*inv(Signal'*Signal);
% Scores=old_X*Signal;
% Scores=X*Signal*inv(Signal'*Signal);

