classdef calibValidPartitioning<dynamicprops
    % Classe permettant le partitionnbement d'un jeu de donnees en 2 sous-jeux Calibration/Validation
    % en tenant compte des replicats analytiques
    %
    % Exemple d'appel:
    % objetPartition = calibValidPartitioning(mydata, firstLetter, LastLetter, calP, valP, firstLetterClass, lastLetterClass);
    %
    % INPUTS:
    % mydata            :   jeu de donnees a diviser en Calibration/Validation au format SAISIR
    % firstLetter       :   index du 1er caractere dans le code-echantillon pour distinguer les replicats
    % LastLetter        :   index du derenier caractere dans le code-echantillon pour distinguer les replicats
    % calP              :   pourcentage d'echantillons a mettre en calibration
    % valP              :   pourcentage d'echantillons a mettre en validation
    % firstLetterClass  :   index du 1er caractere permettant de definir les groupes d'echantillons dans le jeu de donnees
    % lastLetterClass   :   index du dernier caractere permettant de definir les groupes d'echantillons dans le jeu de donnees
    %                  ex.: MED_0001_r1     
    %                       MED_0001_r2
    %                       ...
    %                       BRD_0035_r1
    %                       BRD_0035_r2 
    %                                       firstLetterClass    = 1
    %                                       lastLetterClass     = 3
    %                                       firstLetter         = 5
    %                                       LastLetter          = 8          
    %                                       calP                = 2/3
    %                                       valP                = 1/3
    % RESULTAT en ligne de commande
    %
    % -----------------------------------
    % Nombre moyen de réplicats par échantillon :2
    % -----------------------------------
    % Stats pour les échantillons du groupe :b
    % Calibration - nombre de groupes de réplicats :109
    % Validation - nombre de groupes de réplicats :15
    % -----------------------------------
    % Stats pour les échantillons du groupe :c
    % Calibration - nombre de groupes de réplicats :109
    % Validation - nombre de groupes de réplicats :94

                                            

    properties
        mydata  % jeu de donnees au format SAISIR (.d, .i, .v)
        mygroups
        calibSet = {};
        validSet = {};
        classesEchantillons = {}; % Liste des index correspondants aux différentes classes d'échantillons trouvés. Chaque entrée est une classe d'échantillons avec ses index respectifs.
        existingClassesNames = {}; % Noms arbitraires de classes trouvés selon les index de lettres fournis lors de la création de l'objet.
        nbReplicatsPerGroup = 0;
    end

    methods
        function obj = calibValidPartitioning(mydata, firstLetter, LastLetter, calP, valP, firstLetterClass, lastLetterClass)
            obj.mydata = mydata;
            groupsResults = makeGroups(obj, firstLetter, LastLetter, firstLetterClass, lastLetterClass);
            selectSamples(obj, calP, valP, groupsResults)
        end

        function groupsResults = makeGroups(obj, firstLetter, LastLetter, firstLetterClass, lastLetterClass)
            obj.mygroups = create_group1(obj.mydata, firstLetter, LastLetter);
            groupsNames = cellstr(obj.mygroups.g.i);

            groups = {};
            obj.nbReplicatsPerGroup = 0;
            for i=1:size(groupsNames,1)
                index = find(obj.mygroups.d==i);
                groups{i,1} = selectrow(obj.mydata, index);
                obj.nbReplicatsPerGroup = obj.nbReplicatsPerGroup+size(groups{i,1}.d, 1);
                % Nom du groupe - Pris sur le premier élément des noms ; Par exemple '1_F'
                name = groups{i,1}.i(1,:);
                % Détermination du nom du groupe selon les index fournis
                grpName = name(firstLetterClass:lastLetterClass);
                % Index du groupe testé
                indexG = find(strcmp(obj.existingClassesNames, grpName), 1);
                % Vérification si le groupe n'est pas déjà connu
                if not(isempty(indexG))
                    % Si le groupe est déjà connu, on ajoute l'index (donc
                    % i) à la table correspondante à son groupe. (indexG) 
                    obj.classesEchantillons{indexG}{end+1, 1} = i;
                else
                    % Si le groupe n'existe pas encore, on le créé (avec grpName) puis on
                    % insère son premier élément (donc i)
                    obj.existingClassesNames{end+1, 1} = grpName;
                    obj.classesEchantillons{end+1, 1} = {i};
                end
            end
            groupsResults = groups;
            obj.nbReplicatsPerGroup = obj.nbReplicatsPerGroup/size(groupsResults,1);
        end

        function selectSamples(obj, calP, valP, groupsResults)
            % Effectue la sélection des groupes selon la proportion indiquée.
            % valP : Pourcentage de échantillons à sélectionner pour la partie calul,
            % calP : le reste.

            if nargin < 3
                valP = 1/3;
                calP = 2/3;
            end

            nbGroups = size(groupsResults,1);
            if (valP + calP) ~= 1
                error('Invalid percentages, sum of valP and calP should be equal to 1.');
            end

            % Détermination du nombre de groupe d'échantillons à placer dans le groupe de calibration.
            %tocal = round(nbGroups * calP);

            % Initialisation des groupes de calibration et de validation
            obj.calibSet = {};
            %calibSetNames = {};
            obj.validSet = {};

            %% Détermination du nombre de groupes de réplicats à piocher dans chaque classe.

            % Explication du calcul : on divise le nb total de groupes par
            % le nombre de classes d'échantillons on multiplie ensuite par
            % le pourcentage de groupes de réplicats à mettre dans le set
            % de calibration.
            perClass = round((nbGroups/size(obj.existingClassesNames,1))*calP);
            disp('-----------------------------------')
            disp(strcat('Nombre moyen de réplicats par échantillon :', string(obj.nbReplicatsPerGroup)))
            for e=1:size(obj.existingClassesNames,1)
                disp('-----------------------------------')
                disp(strcat('Stats pour les échantillons du groupe : ',obj.existingClassesNames{e}))
                disp(strcat('Calibration - nombre de groupes de réplicats :', string(perClass)))
                disp(strcat('Validation - nombre de groupes de réplicats :', string(size(obj.classesEchantillons{e},1)-perClass)))
            end
            aux = [];

            for i = 1:size(obj.existingClassesNames,1)
                % Récupération des index correspondants au groupe
                % actuellement traité.
                currentClass = obj.classesEchantillons{i};
                for y = 1:perClass
                    
                    % Tirage d'un nombre pour sélectionner le groupe.
                    selectedGroupIndex = randi(size(currentClass,1)); 

                    % Récupération de l'index réel du groupe
                    group2add = currentClass{selectedGroupIndex};

                    % Ajout de l'ensemble échantillon et ses réplicats à la
                    % liste calibSet et supression de l'index de la liste
                    % de ceux disponibles.
                    obj.calibSet = appendrow1(obj.calibSet, groupsResults{group2add});
                    currentClass(selectedGroupIndex) = [];
                    
                end
                % Une nouvelle boucle est nécessaire pour récupérer les
                % index restants pour les mettre dans la table de
                % validation car on ne peut pas modifier la table
                % groupResults sinon les index contenus dans currentClass
                % ne seront plus valides.
                for z = 1:size(currentClass,1)
                    % Récupération de l'index réél dans la table
                    % groupsResults
                    group2add = currentClass{z};

                    % Ajout de la structure mydata à celle déjà existante
                    % avec la fonction appendrow1
                    aux = appendrow1(aux, groupsResults{group2add});
                end
            end
            obj.validSet = aux;

        end
    end

end

