function [normSpectrum,quotients,medianQuotient] = saisir_pqn(refSpectrum,testSpectra,varargin)
% Function for normalising spectral data by probabilistic quotient
% normalisation [1]. For each spectral variable, the quotient (test_i/ref_i) 
% is calculated and subsequently the median quotient is taken to normalise
% each test spectrum variable against [normalisation factor =
% 1/median(quotient)].
% 
% The input is a reference spectrum (1 x N vector) and M number of test 
% spectra (M x N matrix). The output are M x N normalised spectra, quotients 
% per variable and the median values of the quotient per test spectrum.
% Optionally, a histogram can be generated to visualise the quotient
% distribution for each test spectrum
%
% Syntax
% [normSpectrum,quotients,medianQuotient] = pqn(refSpectrum,testSpectra,'ShowPlot',PlotValue)
% - refSpectrum:        1 x N vector
% - testSpectra:        M x N matrix
% - 'ShowPlot':         true or false (default)
% - normSpectrum:       M x N matrix
% - quotients:          M x N matrix
% - medianQuotients:    M x 1 vector    
%
% References
% [1] F. Dieterle, A. Ross, G. Schlotterbeck, H. Senn, Anal. Chem. 78, 4281–4290 (2006). 
%

% Check if minimum number of input arguments is met
if nargin < 2
   error('Not enough input arguments'); 
end

% Check if reference spectrum is a row vector
if size(refSpectrum,1) > 1
   refSpectrum = refSpectrum';
   if size(refSpectrum,1)
      error('Reference spectrum should be row vector'); 
   end
end

% Check if reference spectrum or spectra are same length as reference
% spectrum
if size(refSpectrum,2) ~= size(testSpectra,2)
   testSpectra = testSpectra';
   % Check if after transposing the lengths are equal
   if size(refSpectrum,2) ~= size(testSpectra,2)
      error('Reference and test spectra do not have same length'); 
   end
end

plotType = 2; % On default no plot of quotient distribution
for j = 1:length(varargin)
   if isequal(varargin{j},'ShowPlot')
       if isequal(varargin{j+1},true)
           plotType = 1;
       end
       if isequal(varargin{j+1},false)
           plotType = 2;
       end
   end
end

% Convert zero values to NaN for quotient calculation
testSpectra(testSpectra==0) = NaN;
refSpectrum(refSpectrum==0) = NaN;

% Calculate quotient per variable
testSpectraCount = size(testSpectra,1);
quotients = []; medianQuotient = [];
for j = 1:testSpectraCount
   quotients(j,:) = testSpectra(j,:)./refSpectrum;
   medianQuotient = [medianQuotient;nanmedian(quotients(j,:))];
   if plotType == 1
      figure('Name',sprintf('Quotient distribution for test spectrum %d',j));
      histogram(quotients(j,:));
      xlabel('Quotient');
      ylabel('Frequency');
      set(gca,'FontName','Gadugi','FontWeight','bold');
      set(gcf,'Color','White');
   end
   normSpectrum(j,:) = testSpectra(j,:)./(1/medianQuotient(j));
end
